@@ -88,6 +88,12 @@ get_provider_group (GoaProvider *_provider)
   return GOA_PROVIDER_GROUP_BRANDED;
 }
 
+static GoaProviderFeatures
+get_provider_features (GoaProvider *_provider)
+{
+  return GOA_PROVIDER_FEATURE_BRANDED | GOA_PROVIDER_FEATURE_CHAT | GOA_PROVIDER_FEATURE_PHOTOS;
+}
+
 /* facebook client flow sends a different auth query then the base
  * OAuth2Provider */
 static gchar *
@@ -132,7 +138,14 @@ get_scope (GoaOAuth2Provider *provider)
     "user_events,"
     "read_mailbox,"
     "xmpp_login,"
-    "email";
+    "email,"
+    "user_photos";
+}
+
+static guint
+get_credentials_generation (GoaProvider *provider)
+{
+  return 1;
 }
 
 static const gchar *
@@ -273,7 +286,26 @@ get_identity_sync (GoaOAuth2Provider  *provider,
 static gboolean
 is_deny_node (GoaOAuth2Provider *provider, WebKitDOMNode *node)
 {
-  return FALSE;
+  WebKitDOMHTMLButtonElement *button_element;
+  gboolean ret;
+  gchar *name;
+
+  name = NULL;
+  ret = FALSE;
+
+  if (!WEBKIT_DOM_IS_HTML_BUTTON_ELEMENT (node))
+    goto out;
+
+  button_element = WEBKIT_DOM_HTML_BUTTON_ELEMENT (node);
+  name = webkit_dom_html_button_element_get_name (button_element);
+  if (g_strcmp0 (name, "cancel_clicked") != 0)
+    goto out;
+
+  ret = TRUE;
+
+ out:
+  g_free (name);
+  return ret;
 }
 
 static gboolean
@@ -317,7 +349,9 @@ build_object (GoaProvider         *provider,
 {
   GoaAccount *account;
   GoaChat *chat = NULL;
+  GoaPhotos *photos = NULL;
   gboolean chat_enabled;
+  gboolean photos_enabled;
   gboolean ret = FALSE;
 
   account = NULL;
@@ -360,6 +394,34 @@ build_object (GoaProvider         *provider,
                         "ChatEnabled");
     }
 
+  /* Photos */
+  photos = goa_object_get_photos (GOA_OBJECT (object));
+  photos_enabled = g_key_file_get_boolean (key_file, group, "PhotosEnabled", NULL);
+
+  if (photos_enabled)
+    {
+      if (photos == NULL)
+        {
+          photos = goa_photos_skeleton_new ();
+          goa_object_skeleton_set_photos (object, photos);
+        }
+    }
+  else
+    {
+      if (photos != NULL)
+        goa_object_skeleton_set_photos (object, NULL);
+    }
+
+  if (just_added)
+    {
+      goa_account_set_photos_disabled (account, !photos_enabled);
+
+      g_signal_connect (account,
+                        "notify::photos-disabled",
+                        G_CALLBACK (goa_util_account_notify_property_cb),
+                        "PhotosEnabled");
+    }
+
   ret = TRUE;
 
  out:
@@ -367,6 +429,7 @@ build_object (GoaProvider         *provider,
     g_object_unref (chat);
   if (account != NULL)
     g_object_unref (account);
+  g_clear_object (&photos);
   return ret;
 }
 
@@ -392,6 +455,11 @@ show_account (GoaProvider         *provider,
                                                    _("Use for"),
                                                    "chat-disabled",
                                                    _("C_hat"));
+
+  goa_util_add_row_switch_from_keyfile_with_blurb (left, right, object,
+                                                   NULL,
+                                                   "photos-disabled",
+                                                   _("_Photos"));
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -401,6 +469,7 @@ add_account_key_values (GoaOAuth2Provider *provider,
                         GVariantBuilder   *builder)
 {
   g_variant_builder_add (builder, "{ss}", "ChatEnabled", "true");
+  g_variant_builder_add (builder, "{ss}", "PhotosEnabled", "true");
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -420,8 +489,10 @@ goa_facebook_provider_class_init (GoaFacebookProviderClass *klass)
   provider_class->get_provider_type          = get_provider_type;
   provider_class->get_provider_name          = get_provider_name;
   provider_class->get_provider_group         = get_provider_group;
+  provider_class->get_provider_features      = get_provider_features;
   provider_class->build_object               = build_object;
   provider_class->show_account               = show_account;
+  provider_class->get_credentials_generation = get_credentials_generation;
 
   oauth2_class = GOA_OAUTH2_PROVIDER_CLASS (klass);
   oauth2_class->get_authorization_uri    = get_authorization_uri;
