@@ -63,6 +63,7 @@ struct _GoaProvider
  * @get_provider_name: Virtual function for goa_provider_get_provider_name().
  * @get_provider_icon: Virtual function for goa_provider_get_provider_icon().
  * @get_provider_group: Virtual function for goa_provider_get_provider_group().
+ * @get_provider_features: Virtual function for goa_provider_get_provider_features().
  * @add_account: Virtual function for goa_provider_add_account().
  * @refresh_account: Virtual function for goa_provider_refresh_account().
  * @build_object: Virtual function for goa_provider_build_object().
@@ -115,10 +116,11 @@ struct _GoaProviderClass
 
   /* pure virtual */
   GoaProviderGroup (*get_provider_group) (GoaProvider   *provider);
+  GoaProviderFeatures  (*get_provider_features)     (GoaProvider   *provider);
 
   /*< private >*/
   /* Padding for future expansion */
-  gpointer goa_reserved[32];
+  gpointer goa_reserved[31];
 };
 
 GType        goa_provider_get_type                  (void) G_GNUC_CONST;
@@ -127,7 +129,12 @@ gchar       *goa_provider_get_provider_name         (GoaProvider         *provid
                                                      GoaObject           *object);
 GIcon       *goa_provider_get_provider_icon         (GoaProvider         *provider,
                                                      GoaObject           *object);
+G_DEPRECATED_FOR(goa_provider_get_provider_features)
 GoaProviderGroup goa_provider_get_provider_group    (GoaProvider         *provider);
+GoaProviderFeatures goa_provider_get_provider_features (GoaProvider      *provider);
+void         goa_provider_set_preseed_data          (GoaProvider *provider,
+                                                     GVariant    *preseed_data);
+GVariant    *goa_provider_get_preseed_data          (GoaProvider *provider);
 GoaObject   *goa_provider_add_account               (GoaProvider         *provider,
                                                      GoaClient           *client,
                                                      GtkDialog           *dialog,
@@ -167,8 +174,8 @@ gboolean     goa_provider_ensure_credentials_sync   (GoaProvider         *provid
                                                      GError             **error);
 guint        goa_provider_get_credentials_generation (GoaProvider        *provider);
 
-
 GList        *goa_provider_get_all (void);
+
 GoaProvider  *goa_provider_get_for_provider_type (const gchar *provider_type);
 
 /* ---------------------------------------------------------------------------------------------------- */
