@@ -2,20 +2,20 @@
 /*
  * Copyright (C) 2012 Red Hat, Inc.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General
+ * Public License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
+ * Boston, MA 02111-1307, USA.
  *
  * Authors: Ray Strode
  */
@@ -57,8 +57,6 @@ struct _GoaKerberosIdentityManagerPrivate
   gboolean is_blocking_scheduler_job;
 
   volatile int pending_refresh_count;
-
-  guint polling_timeout_id;
 };
 
 typedef enum
@@ -119,7 +117,6 @@ G_DEFINE_TYPE_WITH_CODE (GoaKerberosIdentityManager,
                                                 identity_manager_interface_init)
                          G_IMPLEMENT_INTERFACE (G_TYPE_INITABLE,
                                                 initable_interface_init));
-#define FALLBACK_POLLING_INTERVAL 5
 
 static Operation *
 operation_new (GoaKerberosIdentityManager *self,
@@ -763,36 +760,17 @@ get_new_credentials_cache (GoaKerberosIdentityManager *self,
                            krb5_ccache                *credentials_cache)
 {
   krb5_error_code error_code;
-  gboolean supports_multiple_identities;
-
-  if (g_strcmp0 (self->priv->credentials_cache_type, "FILE") == 0)
-    {
-      goa_debug ("GoaKerberosIdentityManager: credential cache type %s doesn't supports cache collections", self->priv->credentials_cache_type);
-      supports_multiple_identities = FALSE;
-    }
-  else if (g_strcmp0 (self->priv->credentials_cache_type, "DIR") == 0 ||
-           g_strcmp0 (self->priv->credentials_cache_type, "KEYRING") == 0)
-    {
-      goa_debug ("GoaKerberosIdentityManager: credential cache type %s supports cache collections", self->priv->credentials_cache_type);
-      supports_multiple_identities = TRUE;
-    }
-  else
-    {
-      goa_debug ("GoaKerberosIdentityManager: don't know if credential cache type %s supports cache collections, assuming yes", self->priv->credentials_cache_type);
-      supports_multiple_identities = TRUE;
-    }
 
   /* If we're configured for FILE based credentials, then we only
    * have one ccache, and we need to use it always.
    *
-   * If we're configured for DIR or KEYRING based credentials, then we
-   * can have multiple ccache's so we should use the default one first
-   * (so it gets selected automatically) and then fallback to unique
-   * ccache names for subsequent tickets.
-   *
+   * If we're configured for DIR based credentials, then we need
+   * to use the default name "tkt" the first time around, and
+   * then need to use unique names for subsequent tickets.
    */
-  if (!supports_multiple_identities ||
-      g_hash_table_size (self->priv->identities) == 0)
+  if (g_strcmp0 (self->priv->credentials_cache_type, "FILE") == 0 ||
+      (g_strcmp0 (self->priv->credentials_cache_type, "DIR") == 0 &&
+       g_hash_table_size (self->priv->identities) == 0))
     {
       krb5_ccache default_cache;
 
@@ -1347,14 +1325,6 @@ on_credentials_cache_changed (GFileMonitor               *monitor,
   schedule_refresh (self);
 }
 
-static gboolean
-on_polling_timeout (GoaKerberosIdentityManager *self)
-{
-  schedule_refresh (self);
-
-  return G_SOURCE_CONTINUE;
-}
-
 static gboolean
 monitor_credentials_cache (GoaKerberosIdentityManager  *self,
                            GError                     **error)
@@ -1362,10 +1332,10 @@ monitor_credentials_cache (GoaKerberosIdentityManager  *self,
   krb5_ccache default_cache;
   const char *cache_type;
   const char *cache_path;
-  GFileMonitor *monitor = NULL;
+  GFile *file;
+  GFileMonitor *monitor;
   krb5_error_code error_code;
-  GError *monitoring_error = NULL;
-  gboolean can_monitor = TRUE;
+  GError *monitoring_error;
 
   error_code = krb5_cc_default (self->priv->kerberos_context, &default_cache);
 
@@ -1389,9 +1359,12 @@ monitor_credentials_cache (GoaKerberosIdentityManager  *self,
 
   if (strcmp (cache_type, "FILE") != 0 && strcmp (cache_type, "DIR") != 0)
     {
-      goa_warning ("GoaKerberosIdentityManager: Using polling for change notification for credential cache type '%s'",
+      g_set_error (error,
+                   GOA_IDENTITY_MANAGER_ERROR,
+                   GOA_IDENTITY_MANAGER_ERROR_UNSUPPORTED_CREDENTIALS,
+                   "Only 'FILE' and 'DIR' credential cache types are really supported, not '%s'",
                    cache_type);
-      can_monitor = FALSE;
+      return FALSE;
     }
 
   g_free (self->priv->credentials_cache_type);
@@ -1415,57 +1388,44 @@ monitor_credentials_cache (GoaKerberosIdentityManager  *self,
   if (cache_path[0] == ':')
     cache_path++;
 
-  if (can_monitor)
+  file = g_file_new_for_path (cache_path);
+
+  monitoring_error = NULL;
+  if (strcmp (cache_type, "FILE") == 0)
     {
-      GFile *file;
-
-      file = g_file_new_for_path (cache_path);
-
-      monitoring_error = NULL;
-      if (strcmp (cache_type, "FILE") == 0)
-        {
-          monitor = g_file_monitor_file (file,
-                                         G_FILE_MONITOR_NONE,
-                                         NULL,
-                                         &monitoring_error);
-        }
-      else if (strcmp (cache_type, "DIR") == 0)
-        {
-          GFile *directory;
-
-          directory = g_file_get_parent (file);
-          monitor = g_file_monitor_directory (directory,
-                                              G_FILE_MONITOR_NONE,
-                                              NULL,
-                                              &monitoring_error);
-          g_object_unref (directory);
-
-        }
-      g_object_unref (file);
+      monitor = g_file_monitor_file (file,
+                                     G_FILE_MONITOR_NONE,
+                                     NULL,
+                                     &monitoring_error);
     }
-
-  if (monitor == NULL)
+  else if (strcmp (cache_type, "DIR") == 0)
     {
-      if (monitoring_error != NULL)
-        {
-          goa_warning ("GoaKerberosIdentityManager: Could not monitor credentials for %s (type %s), reverting to polling: %s",
-                       cache_path,
-                       cache_type,
-                       monitoring_error != NULL? monitoring_error->message : "");
-          g_clear_error (&monitoring_error);
-        }
-      can_monitor = FALSE;
+      GFile *directory;
+
+      directory = g_file_get_parent (file);
+      monitor = g_file_monitor_directory (directory,
+                                          G_FILE_MONITOR_NONE,
+                                          NULL,
+                                          &monitoring_error);
+      g_object_unref (directory);
+
     }
   else
     {
-      self->priv->credentials_cache_changed_signal_id =
-        g_signal_connect (G_OBJECT (monitor), "changed",
-                          G_CALLBACK (on_credentials_cache_changed), self);
-      self->priv->credentials_cache_monitor = monitor;
+      g_assert_not_reached ();
+    }
+  g_object_unref (file);
+
+  if (monitor == NULL)
+    {
+      g_propagate_error (error, monitoring_error);
+      return FALSE;
     }
 
-  if (!can_monitor)
-    self->priv->polling_timeout_id = g_timeout_add_seconds (FALLBACK_POLLING_INTERVAL, (GSourceFunc) on_polling_timeout, self);
+  self->priv->credentials_cache_changed_signal_id =
+    g_signal_connect (G_OBJECT (monitor), "changed",
+                      G_CALLBACK (on_credentials_cache_changed), self);
+  self->priv->credentials_cache_monitor = monitor;
 
   return TRUE;
 }
@@ -1473,19 +1433,11 @@ monitor_credentials_cache (GoaKerberosIdentityManager  *self,
 static void
 stop_watching_credentials_cache (GoaKerberosIdentityManager *self)
 {
-  if (self->priv->credentials_cache_monitor != NULL)
-    {
-      if (!g_file_monitor_is_cancelled (self->priv->credentials_cache_monitor))
-        g_file_monitor_cancel (self->priv->credentials_cache_monitor);
+  if (!g_file_monitor_is_cancelled (self->priv->credentials_cache_monitor))
+    g_file_monitor_cancel (self->priv->credentials_cache_monitor);
 
-      g_clear_object (&self->priv->credentials_cache_monitor);
-    }
-
-  if (self->priv->polling_timeout_id != 0)
-    {
-      g_source_remove (self->priv->polling_timeout_id);
-      self->priv->polling_timeout_id = 0;
-    }
+  g_object_unref (self->priv->credentials_cache_monitor);
+  self->priv->credentials_cache_monitor = NULL;
 }
 
 static gboolean
@@ -1607,7 +1559,8 @@ goa_kerberos_identity_manager_dispose (GObject *object)
       self->priv->identities = NULL;
     }
 
-  stop_watching_credentials_cache (self);
+  if (self->priv->credentials_cache_monitor != NULL)
+    stop_watching_credentials_cache (self);
 
   if (self->priv->pending_operations != NULL)
     cancel_pending_operations (self);
